=== modified file 'hooks/helpers.py'
--- hooks/helpers.py	2012-02-10 21:19:58 +0000
+++ hooks/helpers.py	2012-02-10 21:29:42 +0000
@@ -33,6 +33,7 @@
 import re
 import subprocess
 import sys
+import tempfile
 from textwrap import dedent
 import time
 import urllib2
@@ -202,6 +203,16 @@
         return s
 
 
+def make_charm_config_file(charm_config):
+    charm_config_file = tempfile.NamedTemporaryFile()
+    charm_config_file.write(yaml.dump(charm_config))
+    charm_config_file.flush()
+    # The NamedTemporaryFile instance is returned instead of just the name
+    # because we want to take advantage of garbage collection-triggered
+    # deletion of the temp file when it goes out of scope in the caller.
+    return charm_config_file
+
+
 def unit_info(service_name, item_name, data=None):
     if data is None:
         data = yaml.safe_load(run('juju', 'status'))
@@ -326,8 +337,10 @@
     current_home = os.getenv('HOME')
     home = get_user_home(user)
     os.environ['HOME'] = home
-    yield Env(uid, gid, home)
-    os.setegid(os.getgid())
-    os.seteuid(os.getuid())
-    if current_home is not None:
-        os.environ['HOME'] = current_home
+    try:
+        yield Env(uid, gid, home)
+    finally:
+        os.setegid(os.getgid())
+        os.seteuid(os.getuid())
+        if current_home is not None:
+            os.environ['HOME'] = current_home

=== modified file 'hooks/tests.py'
--- hooks/tests.py	2012-02-08 21:28:34 +0000
+++ hooks/tests.py	2012-02-10 19:43:46 +0000
@@ -1,11 +1,12 @@
-
+import os
+from subprocess import CalledProcessError
 import unittest
-from subprocess import CalledProcessError
 
 from helpers import (
     DictDiffer,
     command,
     run,
+    su,
     unit_info,
     )
 
@@ -136,5 +137,73 @@
         self.assertNotEqual('started', state)
 
 
+current_euid = os.geteuid()
+current_egid = os.getegid()
+current_home = os.environ['HOME']
+example_euid = current_euid + 1
+example_egid = current_egid + 1
+example_home = '/var/lib/example'
+userinfo = {'example_user': dict(
+        ids=(example_euid, example_egid), home=example_home)}
+effective_values = dict(uid=current_euid, gid=current_egid)
+
+
+def stub_os_seteuid(value):
+    effective_values['uid'] = value
+
+
+def stub_os_setegid(value):
+    effective_values['gid'] = value
+
+
+class TestSuContextManager(unittest.TestCase):
+
+    def setUp(self):
+        import helpers
+        self.os_seteuid = os.seteuid
+        self.os_setegid = os.setegid
+        self.helpers_get_user_ids = helpers.get_user_ids
+        self.helpers_get_user_home = helpers.get_user_home
+        os.seteuid = stub_os_seteuid
+        os.setegid = stub_os_setegid
+        helpers.get_user_ids = lambda user: userinfo[user]['ids']
+        helpers.get_user_home = lambda user: userinfo[user]['home']
+
+    def tearDown(self):
+        import helpers
+        os.seteuid = self.os_seteuid
+        os.setegid = self.os_setegid
+        helpers.get_user_ids = self.helpers_get_user_ids
+        helpers.get_user_home = self.helpers_get_user_home
+
+    def testChange(self):
+        with su('example_user'):
+            self.assertEqual(example_euid, effective_values['uid'])
+            self.assertEqual(example_egid, effective_values['gid'])
+            self.assertEqual(example_home, os.environ['HOME'])
+
+    def testEnvironment(self):
+        with su('example_user') as e:
+            self.assertEqual(example_euid, e.uid)
+            self.assertEqual(example_egid, e.gid)
+            self.assertEqual(example_home, e.home)
+
+    def testRevert(self):
+        with su('example_user'):
+            pass
+        self.assertEqual(current_euid, effective_values['uid'])
+        self.assertEqual(current_egid, effective_values['gid'])
+        self.assertEqual(current_home, os.environ['HOME'])
+
+    def testRevertAfterFailure(self):
+        try:
+            with su('example_user'):
+                raise RuntimeError()
+        except RuntimeError:
+            self.assertEqual(current_euid, effective_values['uid'])
+            self.assertEqual(current_egid, effective_values['gid'])
+            self.assertEqual(current_home, os.environ['HOME'])
+
+
 if __name__ == '__main__':
     unittest.main()

=== modified file 'tests/buildbot-master.test'
--- tests/buildbot-master.test	2012-02-10 21:19:58 +0000
+++ tests/buildbot-master.test	2012-02-10 21:29:42 +0000
@@ -4,28 +4,16 @@
 
 from helpers import (
     command,
-    run,
+    make_charm_config_file,
     unit_info,
     wait_for_page_contents,
     wait_for_unit,
     )
 import os.path
-import tempfile
 import unittest
-import yaml
 
 juju = command('juju')
 
-def make_charm_config_file(charm_config):
-    charm_config_file = tempfile.NamedTemporaryFile()
-    charm_config_file.write(yaml.dump(charm_config))
-    charm_config_file.flush()
-    # The NamedTemporaryFile instance is returned instead of just the name
-    # because we want to take advantage of garbage collection-triggered
-    # deletion of the temp file when it goes out of scope in the caller.
-    return charm_config_file
-
-
 def deploy(charm_config):
     charm_config_file = make_charm_config_file(charm_config)
     juju('deploy', 'buildbot-master', '--config='+charm_config_file.name)
@@ -43,7 +31,7 @@
     def test_deploy(self):
         # See if the charm will actual deploy correctly.  This test may be
         # made redundant by standard charm tests run by some future centralized
-        # charm repository.  
+        # charm repository.
         juju('deploy', 'buildbot-master')
         wait_for_unit('buildbot-master')
         self.assertEqual(unit_info('buildbot-master', 'state'), 'started')

